function [ ] = data_storage_policy( )
% -----------------------------------------------------------------------------
%
% NAME
%
%	data_storage_policy
%
% PURPOSE
%
%	This routine computes the data storage policy.
%
% CALLING SEQUENCE
%
%	[ ] = data_storage_policy(  )
%
% INPUT PARAMETERS
% 
%   none
%
% OUTPUT PARAMETERS
%
%   none
%
% HISTORY
%
% 	Version 1
%     1.0 : 2023 May 26 : J. De Keyser
%       Creation from scratch
%
% -----------------------------------------------------------------------------
           
    % Define physical parameters
    all_T = [ 1e4, 2e4, 5e4, 1e5, 2e5, 5e5, 1e6, 5e6, 1e7 ]; % s
    rate_hk = 128; % bps
    rate_lowres = 327; % bps
    rate_reduced = 9792; % bps
    rate_full = 33435; % bps
    margin = 0.20; % 20% margin for data packetization and overhead
    compression_ratio = 1.0; % no on-the-flight compression
    M = 8000 * 1024 * 1024; % bits
        
    % Apply margin and compression
    rate_hk = rate_hk * ( 1 + margin ); % no compression
    rate_lowres = rate_lowres * ( 1 + margin ) / compression_ratio;
    rate_reduced = rate_reduced * ( 1 + margin ) / compression_ratio;
    rate_full = rate_full * ( 1 + margin ) / compression_ratio;

    % Analyze
    for gamma_over_beta = [ 0.1, 0.5, 1 ]
        rate_star = ( rate_reduced + gamma_over_beta * rate_full )/( 1 + gamma_over_beta );
        fprintf( 'gamma/beta = %4.2f\n', gamma_over_beta );
        fprintf( '       T :     alpha    beta    gamma  : M needed \n' );
        for T = all_T

            % Memory left without housekeeping
            Meff = M - T * rate_hk;

            % Average storage rate possible
            rate_ave = Meff / T;

            % Get alpha
            alpha = ( rate_star - rate_ave ) / ( rate_star - rate_lowres );
            if alpha >= 0
                beta = (1-alpha)/(1+gamma_over_beta);
                gamma = beta * gamma_over_beta;
            else
                alpha = 0;
                beta = ( rate_full - rate_ave )/(rate_full - rate_reduced);
                gamma = 1 - beta;
                if gamma > 1
                    gamma = 1;
                    beta = 0;
                end
            end
            Mneeded = (alpha*rate_lowres+beta*rate_reduced+gamma*rate_full+rate_hk)*T;
            fprintf( '  %9.0f  %6.2f  %6.2f  %6.2f : %8.0f\n', T, alpha, beta, gamma, Mneeded );  

        end
    end
    
end

% -----------------------------------------------------------------------------
    
