function [ ] = koenders_model()
% -----------------------------------------------------------------------------
%
% NAME
%
%	koenders_model
%
% PURPOSE
%
%	This routine evaluates Koenders' model at various heliocentric distances.
%
% CALLING SEQUENCE
%
%	[ ] = koenders_model()
%
% INPUT PARAMETERS
% 
%   none
%
% OUTPUT PARAMETERS
%
%   none
%
% HISTORY
%
% 	Version 1
%     1.0 : 2023 May 26 : J. De Keyser
%       Creation from scratch
%
% -----------------------------------------------------------------------------

    koenders_model_Rh( 0.9 );
    pause(3);
    koenders_model_Rh( 1.0 );
    pause(3);
    koenders_model_Rh( 1.2 );
    pause(3);

end

% -----------------------------------------------------------------------------

function [ ] = koenders_model_Rh( Rh )
% -----------------------------------------------------------------------------
%
% NAME
%
%	koenders_model
%
% PURPOSE
%
%	This routine evaluates Koenders' model.
%
% CALLING SEQUENCE
%
%	[ ] = koenders_model( Rh )
%
% INPUT PARAMETERS
% 
%   Rh
%       double: heliocentric distance, in au
%
% OUTPUT PARAMETERS
%
%   none
%
% HISTORY
%
% 	Version 1
%     1.0 : 2023 May 26 : J. De Keyser
%       Creation from scratch
%
% -----------------------------------------------------------------------------
           
    % Define physical parameters
    Qlow = 1.e28; % molecules/s
    Qmax = 1.e32; % molecules/s
    u_n = 1e3; % m/s
    R_n = 2500; % m
    
    % Algorithmic and graphical parameters
    n_Q = 500;
    all_Q = 10.^linspace( log10(Qlow), log10(Qmax), n_Q )';   
    all_Rn_1 = zeros(size(all_Q));
    all_Rn_2 = zeros(size(all_Q));
    all_RbsK = zeros(size(all_Q));
    all_RbsT = zeros(size(all_Q));
    linewidth = 1.;
    fontsize = 14;
    axes_parameters = { 'Box', 'on', 'Layer', 'top', 'Linewidth', linewidth, 'Fontsize', fontsize };
    line_parameters = { 'Linewidth', linewidth };
    text_parameters = { 'Interpreter', 'latex', 'Fontsize', 1.2*fontsize };
    
    % Create windows
    h = findobj( 0, 'Type', 'figure', 'Tag', 'koenders_model' );
    if isempty(h)
        h1 = figure( 'Units', 'pixel', 'Position', [ 50, 50, 800, 600 ], 'Tag', 'koenders_model' );
    else
        h1 = h(1);
        clf(h1);
    end
    ha11 = axes( 'Parent', h1, 'Units', 'normalized', 'Position', [ 0.15, 0.15, 0.80, 0.80 ] );
    hold( ha11, 'on' );
    set( ha11, axes_parameters{:} );
    set( get( ha11, 'YLabel' ), 'String', [ '$R_\mathrm{BS,K}$($', sprintf('%3.1f',Rh), '$\,au) [km]' ], text_parameters{:} );
    set( get( ha11, 'XLabel' ), 'String', '$Q_\mathrm{gas}$($1$\,au) [molecules/s]', text_parameters{:} );
    
    [ n_sw, u_sw, B_sw ] = generate_sw_parameters( Rh );
    for k = 1:length(all_Q)
        Q = all_Q(k)/Rh.^6;
        all_RbsK(k) = bow_shock_standoff( Q, u_n, u_sw, n_sw, B_sw, R_n, Rh );
        all_RbsT(k) = bow_shock_position( all_RbsK(k), 1.0, 0. );
        all_Rn_1(k) = neutral_observable_region( Q, u_n, 10^11 );
        all_Rn_2(k) = neutral_observable_region( Q, u_n, 10^9 );
    end

    Rbsmax = bow_shock_standoff( 1e9*Q, u_n, u_sw, n_sw, B_sw, R_n, Rh );
    
    plot( ha11, all_Q, all_Rn_1/1000, 'r--', line_parameters{:} );
    plot( ha11, all_Q, all_Rn_2/1000, 'r--', line_parameters{:} );
    plot( ha11, all_Q, all_RbsK/1000, 'b-', line_parameters{:} );
    plot( ha11, all_Q, all_RbsT/1000, 'g--', line_parameters{:} );

    plot( ha11, [ Qlow Qmax ], [ 1, 1 ]*Rbsmax/1000, 'k--', line_parameters{:} );
    text( ha11, Qlow*3, 1.4*Rbsmax/1000, '$u_n/\nu$', text_parameters{:} );

    set( ha11, 'XLim', [ Qlow, Qmax ], 'YLim', [ 1e3, 1e7 ] );
    set( ha11, 'XScale', 'log' );
    set( ha11, 'YScale', 'log' );
    set( ha11, 'XGrid', 'on', 'YGrid', 'on' );

    if Rh == 1.0
        legend( ha11, '$R_{n1}$', '$R_{n2}$', '$R_\mathrm{BS,K}$', '$R_\mathrm{BS,K,flank}$', 'Location', 'southeast', text_parameters{:} );
    end

    Rbs_exists = ( all_RbsK > R_n );
    index = find( Rbs_exists, 1, 'first' );
    if index > 1
        ylimits = get( ha11, 'YLim' );
        hp = ...
            patch( ...
                ha11, ...
                'XData', [ Qlow, Qlow, all_Q(index)*0.99, all_Q(index)*0.99 ], ...
                'YData', [ ylimits(1), ylimits(2), ylimits(2), ylimits(1) ], ...
                'ZData', -10*[1 1 1 1], ...
                'FaceColor', 0.85*[1 1 1] ...
            );
    end

    filename = [ 'Figure_Koenders_model', strrep(sprintf('_%3.1f_au', Rh ),'.','p') ];
    print( h1, filename, '-dpdf', '-bestfit' );
    print( h1, filename, '-djpeg' );
        
end

function [ n_sw, u_sw, B_sw ] = generate_sw_parameters( Rh )
    % Constants
    kB = 1.380649e-23;
    % Solar wind speed
    u_sw = 400000;
    % Temperature change with speed
    Tp_sw_1_au = 700*(u_sw/1000)-200000.;
    Te_sw_1_au = Tp_sw_1_au/5.;
    % Magnetic field strength at 1 au
    B_sw_1_au = 7.617e-9;
    % Solar wind plasma beta at 1 au
    beta_1_au = 10.^(-0.2);
    % Obtain density
    Pmag_1_au = B_sw_1_au.^2 / (8*pi*1e-7);
    Pkin_1_au = beta_1_au .* Pmag_1_au;
    n_sw_1_au = Pkin_1_au ./ ( kB * ( Tp_sw_1_au + Te_sw_1_au ) );
    n_sw = n_sw_1_au ./ Rh.^2;
    B_sw = B_sw_1_au * Rh.^(-1.348);
end

function r_BSK = bow_shock_standoff( Q, u_n, u_sw, n_sw, B_sw, R_n, Rh )
    amu = 1.66e-27; % kg
    nu = 7.e-7/Rh.^2; % /s 
    m_i = 19 * amu;
    m_sw = 1 * amu;
    rho_ux_crit = 4/3;
    q_i = 1.60e-19; % C
    Omega_ci  = q_i * B_sw / m_i;
    Omega_ci_times_t_p = 2 * pi * 0.216;
    R_s = ( u_sw / Omega_ci ) * ( Omega_ci_times_t_p - sin( Omega_ci_times_t_p ) );
    A = ( nu * Q * m_i ) / ( 4 * pi * u_n * m_sw * n_sw * u_sw * ( u_n / nu + R_n ) );
    r_BSK = ( nu * Q * m_i ) / ( 4 * pi * u_n * m_sw * n_sw * u_sw * ( rho_ux_crit - 1 + A ) ) - R_s; 
end

function r_BS = bow_shock_position( r_BSK, epsilon, cos_theta )
    r_BS = r_BSK * (1 + epsilon) ./ ( 1 + epsilon * cos_theta );
end

function r_neutral = neutral_observable_region( Q, u_n, n_neutral_limit )
    r_neutral = sqrt( Q / (4 * pi * u_n * n_neutral_limit ) );
end


% -----------------------------------------------------------------------------
    
