function [ ] = science_obs_window( )
% -----------------------------------------------------------------------------
%
% NAME
%
%	science_obs_window
%
% PURPOSE
%
%	This routine determines the science observation window.
%
% CALLING SEQUENCE
%
%	[ ] = science_obs_window(  )
%
% INPUT PARAMETERS
% 
%   none
%
% OUTPUT PARAMETERS
%
%   none
%
% HISTORY
%
% 	Version 1
%     1.0 : 2023 May 26 : J. De Keyser
%       Creation from scratch
%
% -----------------------------------------------------------------------------
           
    % Strategies
    strategies = { 'random', 'biased' };

    % Define physical parameters
    Qlow = 1.e28; % molecules/s
    Qmax = 1.e32; % molecules/s
    u_n = 1e3; % m/s
    alpha = 5.0;
    D_CA = 1000e3; % m
    phi_CA = -30; % degrees
    Rh_all = [ 1.0 ]; % [ 0.9; 1.0; 1.2 ]; % au
    t_B1 = 38*3600; % B1 probe release time in s
    t_B2 = 20*3600; % B2 probe release time in s
    
    % Algorithmic and graphical parameters
    ndays = 100;
    t = linspace( -ndays*24*3600, ndays*24*3600, 2000000 )';    
    n_bins = 80;
    r_low = 1e5;
    r_high = 1e13;
    t_low = 10;
    t_high = 1e8;
    N_v_flyby = 20;
    N_Q = 200;
    N = 5;

    colors = { [ 0.5 0 0 ], [ 0 0.5 0 ], [ 0 0 0.5 ] };
    plot_distribution_range = 1.e-4;
    linewidth = 1.;
    fontsize = 14;
    axes_parameters = { 'Box', 'on', 'Layer', 'top', 'Linewidth', linewidth, 'Fontsize', fontsize };
    line_parameters = { 'Linewidth', linewidth };
    text_parameters = { 'Interpreter', 'latex', 'Fontsize', fontsize };
    legendtext_parameters = { 'Interpreter', 'latex', 'Fontsize', 0.8*fontsize };    

    plot_Q_distributions( Qlow, Qmax, strategies, axes_parameters, line_parameters, text_parameters, colors );

    % Create windows
    h = findobj( 0, 'Type', 'figure', 'Tag', 'science_obs_window-r' );
    if isempty(h)
        h1 = figure( 'Units', 'pixel', 'Position', [ 50, 50, 800, 750 ], 'Tag', 'science_obs_window-r' );
    else
        h1 = h(1);
        clf(h1);
    end
    ha11 = axes( 'Parent', h1, 'Units', 'normalized', 'Position', [ 0.15, 0.70, 0.80, 0.25 ] );
    ha12 = axes( 'Parent', h1, 'Units', 'normalized', 'Position', [ 0.15, 0.40, 0.80, 0.25 ] );
    ha13 = axes( 'Parent', h1, 'Units', 'normalized', 'Position', [ 0.15, 0.10, 0.80, 0.25 ] );
    hold( ha11, 'on' );
    hold( ha12, 'on' );
    hold( ha13, 'on' );
    set( ha11, axes_parameters{:} );
    set( ha12, axes_parameters{:} );
    set( ha13, axes_parameters{:} );
    set( get( ha11, 'YLabel' ), 'String', '$P(R_n)$', text_parameters{:} );
    set( get( ha12, 'YLabel' ), 'String', '$P(R_\mathrm{BS})$', text_parameters{:} );
    set( get( ha13, 'YLabel' ), 'String', '$P(R_\mathrm{ML})$', text_parameters{:} );
    set( get( ha13, 'XLabel' ), 'String', '$R$ [km]', text_parameters{:} );
    
    h = findobj( 0, 'Type', 'figure', 'Tag', 'science_obs_window-t' );
    if isempty(h)
        h2 = figure( 'Units', 'pixel', 'Position', [ 50, 50, 800, 750 ], 'Tag', 'science_obs_window-t' );
    else
        h2 = h(1);
        clf(h2);
    end
    ha21 = axes( 'Parent', h2, 'Units', 'normalized', 'Position', [ 0.15, 0.70, 0.80, 0.25 ] );
    ha22 = axes( 'Parent', h2, 'Units', 'normalized', 'Position', [ 0.15, 0.40, 0.80, 0.25 ] );
    ha23 = axes( 'Parent', h2, 'Units', 'normalized', 'Position', [ 0.15, 0.10, 0.80, 0.25 ] );
    hold( ha21, 'on' );
    hold( ha22, 'on' );
    hold( ha23, 'on' );    
    set( ha21, axes_parameters{:} );
    set( ha22, axes_parameters{:} );
    set( ha23, axes_parameters{:} );
    set( get( ha21, 'YLabel' ), 'String', '$P(t_n)$', text_parameters{:} );
    set( get( ha22, 'YLabel' ), 'String', '$P(t_\mathrm{BS})$', text_parameters{:} );
    set( get( ha23, 'YLabel' ), 'String', '$P(t_\mathrm{ML})$', text_parameters{:} );
    set( get( ha23, 'XLabel' ), 'String', '$t$ [s]', text_parameters{:} );

    legendnames_R_n = {};
    legendnames_R_BS = {};
    legendnames_R_ML = {};
    legendnames_t_n = {};
    legendnames_t_BS = {};
    legendnames_t_ML = {};
    
    for i_strategy = 1:length(strategies)

        for i_Rh = 1:length(Rh_all)
    
            Rh = Rh_all(i_Rh);
    
            fprintf( 'Creating statistics with strategy %s for Rh = %d au ...\n', strategies{i_strategy}, Rh );
        
            all_Q = Q_distribution( Qlow, Qmax, N_Q, strategies{i_strategy} );
                        
            % Assemble statistics
            all_r_neutral1 = [];
            all_r_neutral2 = [];
            all_r_BS1 = [];
            all_r_BS2 = [];
            all_r_ML1 = [];
            all_r_ML2 = [];
            all_t_neutral1 = [];
            all_t_neutral2 = [];
            all_t_BS1 = [];
            all_t_BS2 = [];
            all_t_ML1 = [];
            all_t_ML2 = [];
    
            for k = 1:length(all_Q)
                
                Q = all_Q(k)/Rh^6;
                all_v_flyby = v_distribution( N_v_flyby )*1000; % in m/s
                for i = 1:N_v_flyby
                    v_flyby = all_v_flyby(i);
                    if mod( i, 100 ) == 1
                        fprintf( '.' );
                    end
                    [ r_neutral1, r_neutral2, r_BS1, r_BS2, r_ML1, r_ML2, t_neutral1, t_neutral2, t_BS1, t_BS2, t_ML1, t_ML2 ] = ...
                        perform_statistics_for_given_Q( Q, u_n, alpha, v_flyby, D_CA, phi_CA, Rh, N, t );
                    all_r_neutral1 = [ all_r_neutral1; r_neutral1 ];
                    all_r_neutral2 = [ all_r_neutral2; r_neutral2 ];
                    all_r_BS1 = [ all_r_BS1; r_BS1 ];
                    all_r_BS2 = [ all_r_BS2; r_BS2 ];
                    all_r_ML1 = [ all_r_ML1; r_ML1 ];
                    all_r_ML2 = [ all_r_ML2; r_ML2 ];
                    all_t_neutral1 = [ all_t_neutral1; t_neutral1 ];
                    all_t_neutral2 = [ all_t_neutral2; t_neutral2 ];
                    all_t_BS1 = [ all_t_BS1; t_BS1 ];
                    all_t_BS2 = [ all_t_BS2; t_BS2 ];
                    all_t_ML1 = [ all_t_ML1; t_ML1 ];
                    all_t_ML2 = [ all_t_ML2; t_ML2 ];
                end
                fprintf( '\n' );
    
            end
                
            % Report results - r
            [ all_r_neutral1_bin, all_r_neutral1_counts ] = histogram_1d( all_r_neutral1, n_bins, 'log', r_low, r_high, 'exclusive' );
            [ all_r_neutral2_bin, all_r_neutral2_counts ] = histogram_1d( all_r_neutral2, n_bins, 'log', r_low, r_high, 'exclusive' );
            [ all_r_BS1_bin, all_r_BS1_counts ] = histogram_1d( all_r_BS1, n_bins, 'log', r_low, r_high, 'exclusive' );
            [ all_r_BS2_bin, all_r_BS2_counts ] = histogram_1d( all_r_BS2, n_bins, 'log', r_low, r_high, 'exclusive' );
            [ all_r_ML1_bin, all_r_ML1_counts ] = histogram_1d( all_r_ML1, n_bins, 'log', r_low, r_high, 'exclusive' );
            [ all_r_ML2_bin, all_r_ML2_counts ] = histogram_1d( all_r_ML2, n_bins, 'log', r_low, r_high, 'exclusive' );
            
            all_r_neutral1_counts = all_r_neutral1_counts / ( sum(all_r_neutral1_counts) + 1e-50 ) + 1e-10;
            all_r_neutral2_counts = all_r_neutral2_counts / ( sum(all_r_neutral2_counts) + 1e-50 ) + 1e-10;
            all_r_BS1_counts = all_r_BS1_counts / ( sum(all_r_BS1_counts) + 1e-50 ) + 1e-10;
            all_r_BS2_counts = all_r_BS2_counts / ( sum(all_r_BS2_counts) + 1e-50 ) + 1e-10;
            all_r_ML1_counts = all_r_ML1_counts / ( sum(all_r_ML1_counts) + 1e-50 ) + 1e-10;
            all_r_ML2_counts = all_r_ML2_counts / ( sum(all_r_ML2_counts) + 1e-50 ) + 1e-10;
            
            histoplot( ...
                ha11, ...
                all_r_neutral1_bin/1e3, all_r_neutral1_counts, 'log', '-', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            histoplot( ...
                ha11, ...
                all_r_neutral2_bin/1e3, all_r_neutral2_counts, 'log', '--', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            set( ha11, 'Box', 'on', 'Xscale', 'log', 'Yscale', 'log', 'XLim', [ r_low, r_high ]/1e3, 'YLim', [ plot_distribution_range, 2 ] );
            set( ha11, 'YTick', [ 0.0001 0.001 0.01 0.1 1 ], 'YTickLabel',{ '10^{-4}', '', '10^{-2}', '', '10^{0}' } );
            set( ha11, 'XTickLabel', {} );
            histoplot( ...
                ha12, ...
                all_r_BS1_bin/1e3, all_r_BS1_counts, 'log', '-', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            histoplot( ...
                ha12, ...
                all_r_BS2_bin/1e3, all_r_BS2_counts, 'log', '--', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            set( ha12, 'Box', 'on', 'Xscale', 'log', 'Yscale', 'log', 'XLim', [ r_low, r_high ]/1e3, 'YLim', [ plot_distribution_range, 2 ] );
            set( ha12, 'YTick', [ 0.0001 0.001 0.01 0.1 1 ], 'YTickLabel',{ '10^{-4}', '', '10^{-2}', '', '10^{0}' } );
            set( ha12, 'XTickLabel', {} );
            histoplot( ...
                ha13, ...
                all_r_ML1_bin/1e3, all_r_ML1_counts, 'log', '-', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            histoplot( ...
                ha13, ...
                all_r_ML2_bin/1e3, all_r_ML2_counts, 'log', '--', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            set( ha13, 'Box', 'on', 'Xscale', 'log', 'Yscale', 'log', 'XLim', [ r_low, r_high ]/1e3, 'YLim', [ plot_distribution_range, 2 ] );
            set( ha13, 'YTick', [ 0.0001 0.001 0.01 0.1 1 ], 'YTickLabel',{ '10^{-4}', '', '10^{-2}', '', '10^{0}' } );
        
            % Report results - t
            [ all_t_neutral1_bin, all_t_neutral1_counts ] = histogram_1d( all_t_neutral1, n_bins, 'log', t_low, t_high, 'exclusive' );
            [ all_t_neutral2_bin, all_t_neutral2_counts ] = histogram_1d( all_t_neutral2, n_bins, 'log', t_low, t_high, 'exclusive' );
            [ all_t_BS1_bin, all_t_BS1_counts ] = histogram_1d( -all_t_BS1, n_bins, 'log', t_low, t_high, 'exclusive' );
            [ all_t_BS2_bin, all_t_BS2_counts ] = histogram_1d( all_t_BS2, n_bins, 'log', t_low, t_high, 'exclusive' );
            [ all_t_ML1_bin, all_t_ML1_counts ] = histogram_1d( -all_t_ML1, n_bins, 'log', t_low, t_high, 'exclusive' );
            [ all_t_ML2_bin, all_t_ML2_counts ] = histogram_1d( all_t_ML2, n_bins, 'log', t_low, t_high, 'exclusive' );
            
            all_t_neutral1_counts = all_t_neutral1_counts / ( sum(all_t_neutral1_counts) + 1e-50 ) + 1e-10;
            all_t_neutral2_counts = all_t_neutral2_counts / ( sum(all_t_neutral2_counts) + 1e-50 ) + 1e-10;
            all_t_BS1_counts = all_t_BS1_counts / ( sum(all_t_BS1_counts) + 1e-50 ) + 1e-10;
            all_t_BS2_counts = all_t_BS2_counts / ( sum(all_t_BS2_counts) + 1e-50 ) + 1e-10;
            all_t_ML1_counts = all_t_ML1_counts / ( sum(all_t_ML1_counts) + 1e-50 ) + 1e-10;
            all_t_ML2_counts = all_t_ML2_counts / ( sum(all_t_ML2_counts) + 1e-50 ) + 1e-10;
            
            histoplot( ...
                ha21, ...
                all_t_neutral1_bin, all_t_neutral1_counts, 'log', '-', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            histoplot( ...
                ha21, ...
                all_t_neutral2_bin, all_t_neutral2_counts, 'log', '--', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            set( ha21, 'Box', 'on', 'Xscale', 'log', 'Yscale', 'log', 'XLim', [ t_low, t_high ], 'YLim', [ plot_distribution_range, 2 ] );
            set( ha21, 'YTick', [ 0.0001 0.001 0.01 0.1 1 ], 'YTickLabel',{ '10^{-4}', '', '10^{-2}', '', '10^{0}' } );
            set( ha21, 'XTickLabel', {} );
            histoplot( ...
                ha22, ...
                all_t_BS1_bin, all_t_BS1_counts, 'log', '-', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            histoplot( ...
                ha22, ...
                all_t_BS2_bin, all_t_BS2_counts, 'log', '--', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            set( ha22, 'Box', 'on', 'Xscale', 'log', 'Yscale', 'log', 'XLim', [ t_low, t_high ], 'YLim', [ plot_distribution_range, 2 ] );
            set( ha22, 'YTick', [ 0.0001 0.001 0.01 0.1 1 ], 'YTickLabel',{ '10^{-4}', '', '10^{-2}', '', '10^{0}' } );
            set( ha22, 'XTickLabel', {} );
            histoplot( ...
                ha23, ...
                all_t_ML1_bin, all_t_ML1_counts, 'log', '-', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            histoplot( ...
                ha23, ...
                all_t_ML2_bin, all_t_ML2_counts, 'log', '--', 'Color', colors{i_strategy}, line_parameters{:} ...
            );
            set( ha23, 'Box', 'on', 'Xscale', 'log', 'Yscale', 'log', 'XLim', [ t_low, t_high ], 'YLim', [ plot_distribution_range, 2 ] );        
            set( ha23, 'YTick', [ 0.0001 0.001 0.01 0.1 1 ], 'YTickLabel',{ '10^{-4}', '', '10^{-2}', '', '10^{0}' } );
            
            legendnames_R_n = [ legendnames_R_n, { [ '$R_{n1}$ ', strategies{i_strategy} ], [ '$R_{n2}$ ', strategies{i_strategy} ] } ];
            legendnames_R_BS = [ legendnames_R_BS, { [ '$R_\mathrm{BS1}$ ', strategies{i_strategy} ], [ '$R_\mathrm{BS2}$ ', strategies{i_strategy} ] } ];
            legendnames_R_ML = [ legendnames_R_ML, { [ '$R_\mathrm{ML1}$ ', strategies{i_strategy} ], [ '$R_\mathrm{ML2}$ ', strategies{i_strategy} ] } ];

            legendnames_t_n = [ legendnames_t_n, { [ '$t_{n1}$ ', strategies{i_strategy} ], [ '$t_{n2}$ ', strategies{i_strategy} ] } ];
            legendnames_t_BS = [ legendnames_t_BS, { [ '$t_\mathrm{BS1}$ ', strategies{i_strategy} ], [ '$t_\mathrm{BS2}$ ', strategies{i_strategy} ] } ];
            legendnames_t_ML = [ legendnames_t_ML, { [ '$t_\mathrm{ML1}$ ', strategies{i_strategy} ], [ '$t_\mathrm{ML2}$ ', strategies{i_strategy} ] } ];

            pause( 1. );
        end

    end

    ylimits = get( ha12, 'YLim' );
    plot( ha12, [1,1]*D_CA/1e3, ylimits, 'k:', line_parameters{:} );
    text( ha12, 0.3 * D_CA/1e3, ylimits(2)/10, '$R_\mathrm{CA}$', text_parameters{:} );
    ylimits = get( ha13, 'YLim' );
    plot( ha13, [1,1]*D_CA/1e3, ylimits, 'k:', line_parameters{:} );
    text( ha13, 0.3 * D_CA/1e3, ylimits(2)/10, '$R_\mathrm{CA}$', text_parameters{:} );

    ylimits = get( ha22, 'YLim' );
    plot( ha22, [1,1]*t_B1, ylimits, 'k:', line_parameters{:} );
    text( ha22, 1.1 * t_B1, ylimits(2)/3, 'B1', text_parameters{:} );
    plot( ha22, [1,1]*t_B2, ylimits, 'k:', line_parameters{:} );
    text( ha22, 0.5 * t_B2, ylimits(2)/3, 'B2', text_parameters{:} );
    ylimits = get( ha23, 'YLim' );
    plot( ha23, [1,1]*t_B1, ylimits, 'k:', line_parameters{:} );
    text( ha23, 1.1 * t_B1, ylimits(2)/3, 'B1', text_parameters{:} );
    plot( ha23, [1,1]*t_B2, ylimits, 'k:', line_parameters{:} );
    text( ha23, 0.5 * t_B2, ylimits(2)/3, 'B2', text_parameters{:} );
    
    legend( ha11, legendnames_R_n{:}, 'Location', 'northeast', legendtext_parameters{:} );
    legend( ha12, legendnames_R_BS{:}, 'Location', 'northeast', legendtext_parameters{:} );
    legend( ha13, legendnames_R_ML{:}, 'Location', 'northeast', legendtext_parameters{:} );
    
    legend( ha21, legendnames_t_n{:}, 'Location', 'northeast', legendtext_parameters{:} );
    legend( ha22, legendnames_t_BS{:}, 'Location', 'northeast', legendtext_parameters{:} );
    legend( ha23, legendnames_t_ML{:}, 'Location', 'northeast', legendtext_parameters{:} );

    basename = strrep( sprintf( 'Figure_SciObsWindowAllQ_Strategies_%3.1f_au', Rh ), '.', 'p' );
    print( h1, [ basename, '_r.pdf' ], '-dpdf', '-bestfit' );
    print( h2, [ basename, '_t.pdf' ], '-dpdf', '-bestfit' );
    print( h1, [ basename, '_r.jpg' ], '-djpeg' );
    print( h2, [ basename, '_t.jpg' ], '-djpeg' );
    savefig( h1, [ basename, '_r.fig' ] );
    savefig( h2, [ basename, '_t.fig' ] );
    
end


function [ n_sw, u_sw, B_sw ] = generate_sw_parameters( N, Rh )
    % Constants
    kB = 1.380649e-23;
    % Statistics for solar wind speed
    a = randn( [ N, 1 ] );
    b = randn( [ N, 1 ] );
    c = rand( [ N, 1 ] );
    u_sw_slow = 400000 + 30000*a;
    u_sw_fast = 650000 + 50000*b;
    selection = ( c > 0.75 );
    u_sw = u_sw_slow;
    u_sw(selection) = u_sw_fast(selection);
    % Temperature change with speed
    Tp_sw_1_au = 700*(u_sw/1000)-200000.;
    Te_sw_1_au = Tp_sw_1_au/5.;
    % Statistics for IMF strength at 1 au
    B_sw_1_au_ave = 7.617e-9;
    Pmag_1_au_ave = B_sw_1_au_ave.^2 / (8*pi*1e-7);
    a = randn( [ N, 1 ] );
    Pmag_1_au = Pmag_1_au_ave .* ( 1 + 0.05*a );
    B_sw_1_au = sqrt( 8*pi*1e-7 * Pmag_1_au );
    % Statistics for solar wind plasma beta at 1 au
    a = randn( [ N,1 ] );
    beta_1_au = 10.^(-0.2 + 0.5*a);
    % Obtain density
    Pkin_1_au = beta_1_au .* Pmag_1_au;
    n_sw_1_au = Pkin_1_au ./ ( kB * ( Tp_sw_1_au + Te_sw_1_au ) );
    n_sw = n_sw_1_au ./ Rh.^2;
    B_sw = B_sw_1_au * Rh.^(-1.348);
end

function [ r_neutral1, r_neutral2, r_BS1, r_BS2, r_ML1, r_ML2, t_neutral1, t_neutral2, t_BS1, t_BS2, t_ML1, t_ML2 ] = perform_statistics_for_given_Q( Q, u_n, alpha, v_flyby, D_CA, phi_CA, Rh, N, t )
    [ r, cos_sza ] = trajectory( t, v_flyby, D_CA, phi_CA );
    [ n_sw, u_sw, B_sw ] = generate_sw_parameters( N, Rh );
    r_neutral1 = zeros( [ N, 1 ] );
    r_neutral2 = zeros( [ N, 1 ] );
    r_BS1 = zeros( [ N, 1 ] );
    r_BS2 = zeros( [ N, 1 ] );
    r_ML1 = zeros( [ N, 1 ] );
    r_ML2 = zeros( [ N, 1 ] );
    t_neutral1 = zeros( [ N, 1 ] );
    t_neutral2 = zeros( [ N, 1 ] );
    t_BS1 = zeros( [ N, 1 ] );
    t_BS2 = zeros( [ N, 1 ] );
    t_ML1 = zeros( [ N, 1 ] );
    t_ML2 = zeros( [ N, 1 ] );
    for i = 1:N
        [ r_neutral1(i), r_neutral2(i), r_BS1(i), r_BS2(i), r_ML1(i), r_ML2(i), t_neutral1(i), t_neutral2(i), t_BS1(i), t_BS2(i), t_ML1(i), t_ML2(i) ] = ...
            relevant_distances( Q, u_n, u_sw(i), n_sw(i), B_sw(i), alpha, v_flyby, D_CA, t, r, cos_sza, Rh );
    end
end

function [ r_neutral1, r_neutral2, r_BS1, r_BS2, r_ML1, r_ML2, t_neutral1, t_neutral2, t_BS1, t_BS2, t_ML1, t_ML2 ] = relevant_distances( Q, u_n, u_sw, n_sw, B_sw, alpha, v_flyby, D_CA, t, r, cos_sza, Rh )
    r_neutral1 = neutral_observable_region( Q, u_n, 1e11 );
    r_neutral2 = neutral_observable_region( Q, u_n, 1e9 );
    L1 = sqrt( r_neutral1.^2 - D_CA.^2 );
    t_neutral1 = L1/v_flyby;
    L2 = sqrt( r_neutral2.^2 - D_CA.^2 );
    t_neutral2 = L2/v_flyby;
    r_BSK = bow_shock_standoff( Q, u_n, u_sw, n_sw, B_sw, Rh );
    r_BS = bow_shock_position( r_BSK, cos_sza );
    n = length(r);
    nmid = round(n/2);
    if r(nmid) <= r_BS(nmid)
        % there are intersections
        [ indexl, indexr ] = binary_search_compare( r_BS(1:nmid), r(1:nmid) );
        Fl = r_BS(indexl) - r(indexl);
        Fr = r_BS(indexr) - r(indexr);
        index = ( Fr*indexl - Fl*indexr )/(Fr-Fl);
        r_BS1 = ( (index-indexl)*r_BS(indexr) + (indexr-index)*r_BS(indexl) )/(indexr-indexl);
        t_BS1 = ( (index-indexl)*t(indexr) + (indexr-index)*t(indexl) )/(indexr-indexl);
        [ indexl, indexr ] = binary_search_compare( r_BS(nmid:n), r(nmid:n) );
        indexl = indexl + nmid - 1;
        indexr = indexr + nmid - 1;
        Fl = r_BS(indexl) - r(indexl);
        Fr = r_BS(indexr) - r(indexr);
        index = ( Fr*indexl - Fl*indexr )/(Fr-Fl);
        r_BS2 = ( (index-indexl)*r_BS(indexr) + (indexr-index)*r_BS(indexl) )/(indexr-indexl);
        t_BS2 = ( (index-indexl)*t(indexr) + (indexr-index)*t(indexl) )/(indexr-indexl);
    else
        % there are no intersections
        r_BS1 = 0;
        t_BS1 = 0;
        r_BS2 = 0;
        t_BS2 = 0;
    end
    r_ML = alpha * r_BS;
    if r(nmid) <= r_ML(nmid)
        % there are intersections
        [ indexl, indexr ] = binary_search_compare( r_ML(1:nmid), r(1:nmid) );
        Fl = r_ML(indexl) - r(indexl);
        Fr = r_ML(indexr) - r(indexr);
        index = ( Fr*indexl - Fl*indexr )/(Fr-Fl);
        r_ML1 = ( (index-indexl)*r_ML(indexr) + (indexr-index)*r_ML(indexl) )/(indexr-indexl);
        t_ML1 = ( (index-indexl)*t(indexr) + (indexr-index)*t(indexl) )/(indexr-indexl);
        [ indexl, indexr ] = binary_search_compare( r_ML(nmid:n), r(nmid:n) );
        indexl = indexl + nmid - 1;
        indexr = indexr + nmid - 1;
        Fl = r_ML(indexl) - r(indexl);
        Fr = r_ML(indexr) - r(indexr);
        index = ( Fr*indexl - Fl*indexr )/(Fr-Fl);
        r_ML2 = ( (index-indexl)*r_ML(indexr) + (indexr-index)*r_ML(indexl) )/(indexr-indexl);
        t_ML2 = ( (index-indexl)*t(indexr) + (indexr-index)*t(indexl) )/(indexr-indexl);
    else
        % there are no intersections
        r_ML1 = 0;
        t_ML1 = 0;
        r_ML2 = 0;
        t_ML2 = 0;
    end
end

function [ r, cos_sza ] = trajectory( t, v_flyby, D_CA, phi_CA )
    the_cos = cosd( phi_CA );
    the_sin = sind( phi_CA );
    s = t * v_flyby;
    x = the_cos * D_CA + s * the_sin;
    y = the_sin * D_CA - s * the_cos;
    r = sqrt( x.^2 + y.^2 );
    cos_sza = x ./ r;
end
    
function r_neutral = neutral_observable_region( Q, u_n, n_neutral_limit )
    r_neutral = sqrt( Q / (4 * pi * u_n * n_neutral_limit ) );
end

function r_BSK = bow_shock_standoff( Q, u_n, u_sw, n_sw, B_sw, Rh )
    amu = 1.66e-27; % kg
    R_n = 2500; % m
    nu = 7e-7/Rh.^2; % /s 
    m_i = 19 * amu;
    m_sw = 1 * amu;
    rho_ux_crit = 4/3;
    q_i = 1.60e-19; % C
    Omega_ci  = q_i * B_sw / m_i;
    Omega_ci_times_t_p = 2 * pi * 0.216;
    R_s = ( u_sw / Omega_ci ) * ( Omega_ci_times_t_p - sin( Omega_ci_times_t_p ) );
    A = ( nu * Q * m_i ) / ( 4 * pi * u_n * m_sw * n_sw * u_sw * ( u_n / nu + R_n ) );
    r_BSK = ( nu * Q * m_i ) / ( 4 * pi * u_n * m_sw * n_sw * u_sw * ( rho_ux_crit - 1 + A ) ) - R_s; 
end

function r_BS = bow_shock_position( r_BSK, cos_theta )
    epsilon = 1.0;
    r_BS = r_BSK * (1 + epsilon) ./ ( 1 + epsilon * cos_theta );
end

function [ index_left, index_right ] = binary_search_compare( v1, v2 )
    % return index right before a sign change in v1 - v2
    % it must be given that sign( v1(1)-v2(1) ) * sign( v1(end)-v2(end) ) < 0
    n = length(v1);
    index_left = 1;
    index_right = n;
    dv_left = v1(index_left) - v2(index_left);
    dv_right = v1(index_right) - v2(index_right);
    while index_right - index_left > 1
        index_center = ceil( (index_left+index_right)/2 );
        dv_center = v1(index_center) - v2(index_center);
        if dv_left * dv_center < 0
            index_right = index_center;
            dv_right = dv_center;
        else
            index_left = index_center;
            dv_left = dv_center;
        end
    end
end

function [ vv ] = Q_distribution( Qlow, Qhgh, N, strategy )
    % create distribution for Q

    % assume Halley activity is 10^30 molecules per second
    Qref = 3.42e29;

    Qdistr_low = 1e-4 * 1e30;
    Qdistr_hgh = 1000 * 1e30;

    % table = m0, Activity relative to Halley, frequency per year
    table = [ ...
        1.7,         0.031;
        0.69,        0.082;
        0.28,        0.26;
        0.11,        0.52;
        0.044,       0.76;
        0.017,       1.0;
        0.007,       1.2;
        0.003,       1.5
    ];
    ratio = mean( table(1:end-1,1) ./ table(2:end,1) );
    table_Q_from = table(end:-1:1,1) * Qref;
    table_Q_to = [ table(end-1:-1:1,1); table(1)*ratio ] * Qref;
    table_freq = table(end:-1:1,2);

    % Make a logarithmic table
    table_log_Q_from = log10( table_Q_from );
    table_log_Q_to = log10( table_Q_to );
    table_log_Q = 0.5*( table_log_Q_from + table_log_Q_to );
    table_log_freq = log10( table_freq );
    table_log_freq_density = table_log_freq ./ ( table_log_Q_to-table_log_Q_from );

    % Fit a line through the upper Q half of the table, and use this to
    % extrapolate up to a high mass production which is too dangerous
    % anyhow
    [ a, b ] = fit_line( table_log_Q(end-2:end), table_log_freq_density(end-2:end), ones(3,1) );
    table_log_Q = [ table_log_Q; log10(Qdistr_hgh) ];
    table_log_freq_density = [ table_log_freq_density; a*log10(Qdistr_hgh)+b ];
    % Fit a line through the lower Q half of the table, and use this to
    % extrapolate down to a low mass production which is uninteresting
    % anyhow
    [ a, b ] = fit_line( table_log_Q(1:3), table_log_freq_density(1:3), ones(3,1) );
    table_log_Q = [ log10(Qdistr_low); table_log_Q ];
    table_log_freq_density = [ a*log10(Qdistr_low)+b; table_log_freq_density ];

    % Reinterpolate at higher resolution
    n_bins = 70;
    delta = ( log10(Qdistr_hgh) - log10(Qdistr_low) )/n_bins;
    the_log_Qmid = log10(Qdistr_low) + ( 0.5:n_bins )*delta;
    the_log_Q = linspace(log10(Qdistr_low), log10(Qdistr_hgh), n_bins+1 )';
    the_log_freq_density = interp1(table_log_Q,table_log_freq_density,the_log_Qmid,'linear');

    switch strategy
        case 'random'
            % Constrained within bounds
            the_log_freq_density(the_log_Qmid<log10(Qlow)) = -12;
            the_log_freq_density(the_log_Qmid>log10(Qhgh)) = -12;
        case 'biased'
            % Constrained within bounds, apply linear bias with maximum
            % near Q = 1e29
            the_log_freq_density(the_log_Qmid<log10(Qlow)) = -12;
            the_log_freq_density(the_log_Qmid>log10(Qhgh)) = -12;
            the_log_freq_density = the_log_freq_density+1.0*the_log_Qmid;
    end

    % Normalize distribution
    the_freq_density = 10.^the_log_freq_density;
    the_freq = the_freq_density/(sum(the_freq_density));

    % cumulative distribution
    the_cumfreq = [ 0, cumsum( the_freq ) ];
    % uniform random variables
    s = rand( [ N, 1 ] );
    % obtain abscissae
    vv = interp1( the_cumfreq, the_log_Q, s );
    vv = 10.^vv;
end

function [ vv ] = v_distribution( N )
    % create distribution for v_flyby
    % velocity range
    v = linspace(10,70,50);
    % rescale to [ 0, 1 ]
    x = (v-10)/60;
    % beta distribution
    f = x.^6 .* (1-x).^3 + 1e-14;
    % cumulative distribution
    ff = cumsum(f);
    ff = ff/ff(end);
    % uniform random variables
    s = rand( [ N, 1 ] );
    % obtain abscissae
    vv = interp1( ff, v, s );
end

function [] = histoplot( ha, t, f, scale, varargin )
    [ tf, ff ] = boxplot( t, f, scale );
    plot( ha, tf, ff, varargin{:} );
end

function [ tf, yf ] = boxplot( t, f, scale )
    zeta = 20;
    n = length(t);
    switch scale
        case 'linear'
            % do nothing
        case 'log'
            t = log(t);
    end
    dt = t(2)-t(1);
    nf = n*zeta;
    tf = linspace(t(1)-0.5*dt,t(n)+0.5*dt,nf)';
    yf = interp1( t, f, tf, 'nearest', 'extrap' );
    switch scale
        case 'linear'
            % do nothing
        case 'log'
            tf = exp(tf);
    end
end

function [] = plot_Q_distributions( Qlow, Qhgh, strategies, axes_parameters, line_parameters, text_parameters, colors )
    % Create window
    h = findobj( 0, 'Type', 'figure', 'Tag', 'science_obs_window_Q_distribution' );
     if isempty(h)
        h1 = figure( 'Units', 'pixel', 'Position', [ 50, 50, 600, 450 ], 'Tag', 'science_obs_window_Q_distribution' );
    else
        h1 = h(1);
        clf(h1);
    end
    ha1 = axes( 'Parent', h1, 'Units', 'normalized', 'Position', [ 0.20, 0.20, 0.70, 0.60 ] );
    hold( ha1, 'on' );
    set( ha1, axes_parameters{:} );
    set( get( ha1, 'YLabel' ), 'String', '$P(Q_\mathrm{gas}(1$\,au$))$', text_parameters{:} );
    set( get( ha1, 'XLabel' ), 'String', '$\mathrm{log}_{10}Q_\mathrm{gas}$($1$\,au) [molecules/s]', text_parameters{:} );
    % Strategies
    for i = 1:length(strategies)
        % Generate the distribution
        Q = Q_distribution( Qlow, Qhgh, 100000, strategies{i} );
        % Make statistics
        [ all_bin, all_counts ] = histogram_1d( log10(Q), 50, 'linear', log10(0.1*Qlow), log10(1.0*Qhgh), 'exclusive' );
        all_counts = all_counts/sum(all_counts);
        histoplot( ha1, all_bin, all_counts, 'linear', line_parameters{:}, 'Color', colors{i}, 'DisplayName', strategies{i} );
    end
    set( ha1, 'YLim', [ 0, 0.22 ] );
    legend;

    print( h1, 'Figure_Q_distributions.pdf', '-dpdf', '-bestfit' );
    print( h1, 'Figure_Q_distributions.jpg', '-djpeg' );
    
end


% -----------------------------------------------------------------------------

function [ a, b ] = fit_line( x, f, df )
% -----------------------------------------------------------------------------
%
% NAME
%
%	fit_line
%
% PURPOSE
%
%	This program fits a line to the data.
%   The fit is of the form
%       f = a*x + b
%   A weighted least squares fit is used, with df being the errors on the
%   data so that 1/df^2 are the weights.
%
% CALLING SEQUENCE
%
%	[ a, b ] = fit_line( x, f, df )
%
% INPUT PARAMETERS
%
%   x
%     abscissae, double, vector of size (n,1)
%     
%   f, df
%     function values and errors, double, vector of size (n,1)
%
% OUTPUT PARAMETERS
%
%   a, b
%     double, coefficients
%
% -----------------------------------------------------------------------------

    % Prepare
    x2 = x.*x;
    df2 = df.^2;
    sum_x0 = sum(1./df2);
    sum_x1 = sum(x./df2);
    sum_x2 = sum(x2./df2);
    sum_fx0 = sum(f./df2);
    sum_fx1 = sum(f.*x./df2);
    
    % Solve the system
    A = [ sum_x2, sum_x1; sum_x1, sum_x0 ];
    B = [ sum_fx1; sum_fx0 ];
    coeff = A\B;
    
    % Extract coefficients
    a = coeff(1);
    b = coeff(2);    
    
end
   

% =============================================================================

function [ x_bin, counts ] = histogram_1d( x, n_bins_x, scale_x, varargin )
% -----------------------------------------------------------------------------
%
% NAME
%
%	histogram_1d
%
% PURPOSE
%
%	This routine constructs a histogram of a monovariate distributed quantity.
%
% CALLING SEQUENCE
%
%	[ x_bin, counts ] = histogram_1d( x, n_bins_x, scale_x )
%	[ x_bin, counts ] = histogram_1d( x, n_bins_x, scale_x, x_low, x_high, x_mode )
%
% INPUT PARAMETERS
%
%   x
%     abscissa values
%
%   n_bins_x
%     number of data bins
%
%   scale_x
%     string, 'linear' or 'log'
%
%   x_low, x_high
%     range over which to construct the histogram
%
%   x_mode
%     string, mode of dealing with variable values outside the given
%     interval, 'exclusive' or 'inclusive'
%
% OUTPUT PARAMETERS
%
%   x_bin
%     abscissa binning interval midpoints
%
%   counts
%     counts in each bin
%
% -----------------------------------------------------------------------------

    % return empty histogram for empty input
    if isempty(x) || ( n_bins_x <= 0 )
        x_bin = [];
        counts = [];
        return
    end
    
    % set binning limits
    if isempty(varargin)
        % set binning limits at extremes
        x_low = min(x);
        x_high = max(x);
        % now all x guaranteed between x_low and x_high
    else
        % set binning limits as given
        x_low = varargin{1};
        x_high = varargin{2};
        x_mode = varargin{3};
        % guarantee that all x are between x_low and x_high
        switch x_mode
            case 'inclusive'
                % include out of range values in the two extreme bins
                x( x < x_low ) = x_low;
                x( x > x_high ) = x_high;
            case 'exclusive'
                % ignore values outside range
                x = x( ( x >= x_low ) & ( x <= x_high ) );
        end
    end
    
    % transform
    switch scale_x
        case 'log'
            x = log(x);
            x_low = log(x_low);
            x_high = log(x_high);
        % case 'linear'
            % x, x_low, x_high not changed
    end
    
    % provide margin
    x_range = x_high - x_low;
%     x_low = x_low - 1.e-14*x_range;
%     x_high = x_high + 1.e-14*x_range;
%     x_range = x_high - x_low;
    
    % determine bin centers
    dx = x_range/n_bins_x;
    x_bin = x_low + ( 0.5 : n_bins_x ) * dx;
    switch scale_x
        case 'log'
            x_bin = exp(x_bin);
        % case 'linear'
            % x_bin not changed
    end
    
    n = length(x);
    counts = zeros(n_bins_x,1);
    
    ibin = floor( ( x - x_low ) / ( dx*(1+eps) ) ) + 1;
    for i = 1:n
        idx = ibin(i);
        counts(idx) = counts(idx) + 1;
    end

end

% =============================================================================


